function [V_noise,V_EEG, V_alpha,V_delta,V_theta,V_beta, time] = func_EEGgenerationMPAmodel(EOG,fs)

% this function is the simulated EEG generation based on the MPA model
%   EOG - amplitude of the EOG artifact
%   fs  sampling rate, the time interval is 1/fs
%   V_noise - simulated EEG with artifacts
%   V_alpha - simulated alpha rhythm
%   time - the time vector 

%% the real EEG used to determine the parameters in the MPA model
y = xlsread('close2.xlsx'); % real EEG data

% the power spectrum of the real EEG
N=length(y);
time=(0:N-1)/fs;
Nfft=2048; 
window=kaiser(512,3);   
noverlap=256;   
[Pxx,fp]=pwelch(y,window,noverlap,Nfft,fs);
for i=1:10
    Pxx(i) = 0;
end


%% obtain mk: peak frequency of the power spectrum, Hk: the amplitude of the power spectrum , Fk: frequency width at
% half of Hk in delta rhythm (1 4), theta4 8, alpha8 13) and beta13 30

% obtain the parameters of delta rhythm m1, H1, F1
[maxr,indexr]=max(Pxx(10:41));
H1 = maxr;          
m1 = fp(indexr+9);
j=1;
for i=10:indexr+9
    if (Pxx(i)>=H1/2 && Pxx(i-1)<H1/2)
        f11(j)=fp(i);
        j = j+1;
    end
end
f1_1 = max(f11);
j=1;
for i =indexr+1:50
    if (Pxx(i)<=H1/2 && Pxx(i-1)>H1/2)
        f12(j)=fp(i);
        j = j+1;
    end
end
f1_2 = min(f12);
F1 = f1_2 - f1_1;

% obtain the parameters of theta rhythm m2, H2, F2
[maxr,indexr]=max(Pxx(51:82));
H2 = maxr;
m2 = fp(indexr+50);
j=1;
for i=40:indexr+50
    if (Pxx(i)>=H2/2 && Pxx(i-1)<H2/2)
        f21(j)=fp(i);
        j = j+1;
    end
end
f2_1 = max(f21);
j=1;
for i =indexr+1+50:90
    if (Pxx(i)<=H2/2 && Pxx(i-1)>H2/2)
        f22(j)=fp(i);
        j = j+1;
    end
end
f2_2 = min(f22);
F2 = f2_2 - f2_1;

% obtain the parameters of alpha rhythm m3, H3, F3
[maxr,indexr]=max(Pxx(83:133));
H3 = maxr;
m3 = fp(indexr+82);
j=1;
for i=80:indexr+82
    if (Pxx(i)>=H3/2 && Pxx(i-1)<H3/2)
        f31(j)=fp(i);
        j = j+1;
    end
end
f3_1 = max(f31);
j=1;
for i =indexr+1+82:140
    if (Pxx(i)<=H3/2 && Pxx(i-1)>H3/2)
        f32(j)=fp(i);
        j = j+1;
    end
end
f3_2 = min(f32);
F3 = f3_2 - f3_1;

% obtain the parameters of beta rhythm m4, H4, F4
[maxr,indexr]=max(Pxx(134:256));
H4 = maxr;
m4 = fp(indexr+133);
j=1;
for i=130:indexr+133
    if (Pxx(i)>=H4/2 && Pxx(i-1)<H4/2)
        f41(j)=fp(i);
        j = j+1;
    end
end
f4_1 = max(f41);
j=1;
for i =indexr+1+133:260
    if (Pxx(i)<=H4/2 && Pxx(i-1)>H4/2)
        f42(j)=fp(i);
        j = j+1;
    end
end
f4_2 = min(f42);
F4 = f4_2 - f4_1;

% calculate the parameters of the MPA model: gamak,sigmak
F = [F1 F2 F3 F4];
H = [H1 H2 H3 H4];
for i=1:4
    gama(i)= 2-cos(pi*F(i)/fs)-((2-cos(pi*F(i)/fs))^2-1)^0.5; % gamak
    sigma(i)=(4*H(i)*fs*(1-gama(i))^2)^0.5;                   % sigmak
end



%% simulate the delta, theta, alpha and beta rhythms according to the MPA
% model parameters: gamak, sigmak

% simulate the delta rhythm based on the amplitude of the first-order Markov process
ksi1(1)=normrnd(0,sigma(1));
a1(1)= 0;
V_delta(1) = 0;
for i=2:N
    a1(i)=gama(1)*a1(i-1)+ksi1(i-1);
    ksi1(i)= normrnd(0,sigma(1));
    V_delta(i) = a1(i)*sin(2*pi*m1*(i-1)/fs);
end
               
% simulate the theta rhythm based on the amplitude of the first-order
% Markov process
ksi2(1)=normrnd(0,sigma(2));
a2(1)= 0;
V_theta(1) = 0;
for i=2:N
    a2(i)=gama(2)*a2(i-1)+ksi2(i-1);
    ksi2(i)= normrnd(0,sigma(2));
    V_theta(i) = a2(i)*sin(2*pi*m2*(i-1)/fs);
end

% simulate the alpha rhythm based on the amplitude of the first-order
% Markov process
ksi3(1)=normrnd(0,sigma(3));
a3(1)= 0;
V_alpha(1) = 0;
for i=2:N
    a3(i)=gama(3)*a3(i-1)+ksi3(i-1);
    ksi3(i)= normrnd(0,sigma(3));
    V_alpha(i) = a3(i)*sin(2*pi*m3*(i-1)/fs);
end

% simulate the beta rhythm based on the amplitude of the first-order
% Markov process
ksi4(1)=normrnd(0,sigma(4));
a4(1)= 0;
V_beta(1) = 0;
for i=2:N
    a4(i)=gama(4)*a4(i-1)+ksi4(i-1);
    ksi4(i)= normrnd(0,sigma(4));
    V_beta(i) = a4(i)*sin(2*pi*m4*(i-1)/fs);
end

% simulated spontaneous EEG
V_EEG = V_delta+V_theta+V_alpha+V_beta;

% theoretical power spectrum of the simulated spontaneous EEG
for i=1:length(fp)
        P(i)=0.25*sigma(1)^2/fs/(1+gama(1)^2-2*gama(1)*cos(2*pi*(fp(i)-m1)/fs)) +0.25*sigma(1)^2/fs/(1+gama(1)^2-2*gama(1)*cos(2*pi*(fp(i)+m1)/fs)) +0.25*sigma(2)^2/fs/(1+gama(2)^2-2*gama(2)*cos(2*pi*(fp(i)-m2)/fs))+0.25*sigma(2)^2/fs/(1+gama(2)^2-2*gama(2)*cos(2*pi*(fp(i)+m2)/fs)) +0.25*sigma(3)^2/fs/(1+gama(3)^2-2*gama(3)*cos(2*pi*(fp(i)-m3)/fs))+0.25*sigma(3)^2/fs/(1+gama(3)^2-2*gama(3)*cos(2*pi*(fp(i)+m3)/fs)) +0.25*sigma(4)^2/fs/(1+gama(4)^2-2*gama(4)*cos(2*pi*(fp(i)-m4)/fs))+0.25*sigma(4)^2/fs/(1+gama(4)^2-2*gama(4)*cos(2*pi*(fp(i)+m4)/fs));
end


%% simulated aritfacts and noise

% white Gaussian noise
V_Wnoise = wgn(1, N, 15);  

% EOG artifact, a triangular pulse simulation a triangular pulse with the period set to 3 seconds and the pulse width set to 0.3 seconds 
Nrf=0.3*fs/2;                          
V_EOG=linspace(0,0,N);                
V_EOG(1:Nrf)=(EOG/Nrf)*(1:Nrf);                    
V_EOG(3*fs+(1:Nrf))=(EOG/Nrf)*(1:Nrf);
V_EOG(6*fs+(1:Nrf))=(EOG/Nrf)*(1:Nrf);
V_EOG(Nrf+(1:Nrf))=EOG-(EOG/Nrf)*(1:Nrf);           
V_EOG(3*fs+Nrf+(1:Nrf))=EOG-(EOG/Nrf)*(1:Nrf);
V_EOG(6*fs+Nrf+(1:Nrf))=EOG-(EOG/Nrf)*(1:Nrf);

% baseline drift artifact with amplitude set to 10 V and frequency set to 0.5 Hz 
shift=10;
V_shift=shift*sin(2*pi*0.5*time);     

% the simulated EEG with artifacts
V_noise = V_EEG+V_Wnoise+V_EOG+V_shift;


%% figure

% power spectrum of real and simulated EEG
figure   
plot(fp,Pxx,fp,P,'-r');
xlim([0 30])
xlabel('Frequency (Hz)');ylabel('Power Spectrum (uV^2/Hz)');
legend('Real EEG','Simulated EEG');

% simulated delta, theta, alpha and beta rhythm
figure
subplot(4,1,1)
plot(time,V_delta)
ylim([-100 100])
ylabel('uV');
title('Delta Rhythm'); 
subplot(4,1,2)
plot(time,V_theta)
ylim([-100 100])
ylabel('uV');
title('Theta Rhythm'); 
subplot(4,1,3)
plot(time,V_alpha)
ylim([-100 100])
ylabel('uV');
title('Alpha Rhythm'); 
subplot(4,1,4)
plot(time,V_beta)
ylim([-100 100])
xlabel('Time (s)');ylabel('uV');
title('Beta Rhythm'); 

% simulated spontaneous EEG 
figure
plot(time,V_EEG)
ylim([-100 100])
xlabel('Time (s)');ylabel('Amplitude (uV)');
title('Simulated EEG Time Series'); 


