function [alpha_ssa] = func_rhythmsextraction(RC,D_s,dmax,fs)

% this function is the alpha rhythm extraction based on the grouping rule
%   RC - reconstructed components
%   D_s - eigenvalues
%   dmax  amplitude of the EEG signal
%   fs - sampling rate
%   alpha_ssa - the extracted alpha rhythm

% obtain the length of data (N) and number of elementary matrices (comp)
[N comp] = size(RC);
% frequencies in the FFT
f=(0:N-1)*fs/N;    

if dmax >200    % large artifacts
    num_artifacts=2;
    j=0;
    com = zeros(1,comp);
    
    % the grouping criteria for the eigenvalue pairs of periodic components
    for i=num_artifacts+1:comp-1
        if D_s(i+1)/D_s(i)>0.95
            com(i)=1;
            j=j+1;
        end
    end
    RCs_num=comp-j-num_artifacts; % number of periodic components and the remaining non-periodic RC
       
    % to obtain RCs: the periodic components and the remaining non-periodic RC
    j=1;    
    RCs=zeros(N,RCs_num);
    for i=num_artifacts+1:comp        
        if ((i==num_artifacts+1 && com(i)==1 &&  com(i+1)==0) ||(i>num_artifacts+1 && com(i)==1 && com(i-1)==0 && com(i+1)==0))      
            RCs(:,j)=RC(:,i)+RC(:,i+1); 
            j=j+1;            
        elseif (com(i)==1 && com(i+1)==1)    
            RCs(:,j)=RC(:,i)+RC(:,i+1)+RC(:,i+2); 
            j=j+1;            
        elseif ((i==num_artifacts+1 && com(i)==0)||(i>num_artifacts+1 && com(i)==0 && com(i-1)==0))        
            RCs(:,j)=RC(:,i); 
            j=j+1;
        end
    end
    
    % peak frequencies in FFT of RCs
    for i=1:RCs_num
        y_RCs=abs(fft(RCs(:,i)))*2/N;
        [maxr,indexr]=max(y_RCs(1:round(N/2)));
        f_max(i)=f(indexr);  % peak frequencies
    end
       
    
else    % small artifacts
    j=0;
    com = zeros(1,comp);
    for i=2:comp-1
        if D_s(i+1)/D_s(i)>0.95
            com(i)=1;
            j=j+1;
        end
    end
    RCs_num=comp-j-1;
        
    j=1;
    RCs=zeros(N,RCs_num);
    for i=2:comp
        
        if ((i==2 && com(i)==1 &&  com(i+1)==0) ||(i>2 && com(i)==1 && com(i-1)==0 && com(i+1)==0))      
            RCs(:,j)=RC(:,i)+RC(:,i+1); 
            j=j+1;            
        elseif (com(i)==1 && com(i+1)==1)    
            RCs(:,j)=RC(:,i)+RC(:,i+1)+RC(:,i+2); 
            j=j+1;            
        elseif ((i==2 && com(i)==0)||(i>2 && com(i)==0 && com(i-1)==0))        
            RCs(:,j)=RC(:,i); 
            j=j+1;
        end
    end
    
    for i=1:RCs_num
        y_RCs=abs(fft(RCs(:,i)))*2/N;
        [maxr,indexr]=max(y_RCs(1:round(N/2)));
        f_max(i)=f(indexr);  
    end 
    
end

% extracted alpha rhythm (alpha_ssa)
alpha_ssa=zeros(N,1);
for i=1:RCs_num
    if 8<f_max(i)&&f_max(i)<=13
        alpha_ssa=alpha_ssa+RCs(:,i);  
    end
end
