% this program is the formulation of the adaptive SSA for artifacts removal
% and rhythms extraction of simulated EEG signal 

clc;clear;close all

%% the parameters in the program
fs=200;            % the sampling rate fs = 200Hz
L = 40;            % window length of embedding L = 40
comp=9;            % the number of matrices in the SSA
EOG = 50;         % amplitude of the EOG artifact in the EEG simulation

%% the main program
% simulate the EEG based on the MPA model
% [V_noise,V_EEG, V_alpha,V_delta,V_theta,V_beta, time] = func_EEGgenerationMPAmodel(EOG,fs);
load('V_noise.mat'); load('V_EEG.mat');load('V_alpha.mat');
N=length(V_noise); % number of samples or the length of the data
time=(0:N-1)/fs;

%%  artifacts removal
dmax = max(abs(V_noise)); % the amplitude of the EEG signal

% embeding step in the SSA to map the trajectory matrix X
[ X ] = func_embedding( V_noise, L);

% SVD step in the SSA to obtain the eigenvalues D_s and elementary matrices Xi
[D_s,Xi] = func_svd( X,comp);

% the reconstruction stage to obtain the reconstructed components RC
[ RC ] = func_reconstruction (Xi, N);

% artifact removalto obtian the artifacts: artifact_ssa
[artifact_ssa ]=func_artifactsremoval(RC,dmax);
sig_ssa = V_noise' - artifact_ssa; % corrected EEG

%% CSSA


% 켣
% embeding step in the SSA to map the trajectory matrix X
[ X ] = func_embedding( sig_ssa', L);

% ֽ
[XB]=func_matrix_decompositon(sig_ssa',X,L,N);

% Խƽ
% the reconstruction stage to obtain the reconstructed components RC
[ RC ] = func_reconstruction (XB, N);

%% RCƵ
% RCsfft
col=ceil((6/2));
f=(0:N-1)*fs/N;
figure
for i=1:6
   y_RC=abs(fft(RC(:,i)))*2/N;
   [maxr,indexr]=max(y_RC(1:N/2));
   f_max(i)=f(indexr);
   q = num2str(i);
   t=['RC', q, ' FFT'];
   subplot(col,2,i)
   plot(f(1:N/2),y_RC(1:N/2)) 
   grid on
   title(t);
   xlabel('f/Hz')
   ylabel('Amplitude')
end

% RCĹ
col=ceil((6/2));
f=(0:N-1)*fs/N;
Nfft=2048; 
window=kaiser(512,3);   
noverlap=256;   
figure
for i=1:6   
    [Pp,fp]=pwelch(RC(:,i),window,noverlap,Nfft,fs);
%     for k=1:10
%         Pp(k) = 0;
%     end
   [maxr,indexr]=max(Pp);
   f_max(i)=fp(indexr);
   ff=((i-1)*fs/L)*ones(length(Pp),1);
   pf=(max(Pp)/length(Pp))*(1:length(Pp));
   q = num2str(i);
   t=['RC', q, ' PSD'];
   subplot(col,2,i)
   plot(fp,Pp,ff,pf,'--r') 
   grid on
   title(t);
   xlim([0 60])
   xlabel('f/Hz')
   ylabel('PSD(uV^2/Hz)')
end


% power spectrum of extracted and simulated alpha
[Pe,fp]=pwelch(RC(:,3),window,noverlap,Nfft,fs);
[Ps,fp]=pwelch(V_alpha,window,noverlap,Nfft,fs);
figure   
plot(fp,Pe,fp,Ps,'-r');
xlim([0 30])
xlabel('Frequency (Hz)');ylabel('Power Spectrum (uV^2/Hz)');
legend('Extracted alpha','Simulated alpha');
 
rangeL=20:20:200;
[error]=func_error_alpha(sig_ssa',rangeL,Ps,fs);
figure
plot(rangeL,error,'.-')
ylim([0.15 0.35]);
xlabel('Embedding demension');ylabel('Error (uV^2/Hz)');

% simulated spontaneous EEG 
figure
plot(time,sig_ssa)
ylim([-100 100])
xlabel('Time (s)');ylabel('Amplitude (uV)');
title('Corrected EEG Time Series'); 


